{*******************************************************}
{                                                       }
{  VG Library II                                        }
{                                                       }
{  Picture Mask Editor                                  }
{                                                       }
{  (c) 1997, 2001 by Vladimir Gaitanoff                 }
{                                                       }
{*******************************************************}

{$I vg2.inc}
{$D-,L-}

unit vg2MaskDs;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  vg2SimpleDlg, ExtCtrls, vg2ExtCtrls, StdCtrls, ComCtrls, vg2StdCtrls,
  vg2PropStore, IniFiles, vg2MaskEd, vg2BaseMaskEd;

type
  Tvg2MaskEditorForm = class(Tvg2BaseMaskEditDialogForm)
    edTest: Tvg2Edit;
    Label1: TLabel;
    lvSamples: TListView;
    cmInsert: TButton;
    cmEdit: TButton;
    cmDelete: TButton;
    Label3: TLabel;
    Label4: TLabel;
    lbStatus: TLabel;
    procedure FormShow(Sender: TObject);
    procedure edTestEnter(Sender: TObject);
    procedure edTestChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure cmEditClick(Sender: TObject);
    procedure cmDeleteClick(Sender: TObject);
    procedure lvSamplesClick(Sender: TObject);
  private
    { Private declarations }
    FPicture: Tvg2PictureMask;
    FIniFile: TCustomIniFile;
    FModified: Boolean;
    procedure AssignPicture(APicture: Tvg2PictureMask);
    procedure AssignListItem(Item: TListItem; const APicture, ADescription: string; AFlags: Integer);
    procedure UpdateStatus;
    procedure UpdateStates;
    procedure LoadMasks;
    procedure SaveMasks;
    procedure Modified;
  public
    { Public declarations }
  end;

function EditPicture(APicture: Tvg2PictureMask; AIniFile: TCustomIniFile = nil): Boolean;

const
  SPictures = 'Pictures';
  SCount = 'Count';
  SPicture = 'Picture%d';
  SDescription = 'Descr%d';
  SFlags = 'Flags%d';

implementation
uses vg2Consts, vg2SysUtils, vg2Mask;

{$R *.DFM}

function EditPicture(APicture: Tvg2PictureMask; AIniFile: TCustomIniFile = nil): Boolean;
begin
  with Tvg2MaskEditorForm.Create(nil) do
  try
    FPicture := APicture;
    FIniFile := AIniFile;
    Result := ShowModal = mrOK;
    if Result then
      AssignPicture(APicture);
  finally
    Free;
  end;
end;

procedure Tvg2MaskEditorForm.AssignPicture(APicture: Tvg2PictureMask);
begin
  APicture.PictureMask := edMask.Text;
  if chAutoFill.Checked then
    APicture.Options := APicture.Options + [poAutoFill] else
    APicture.Options := APicture.Options - [poAutoFill];
  if chCaseCorrection.Checked then
    APicture.Options := APicture.Options + [poCaseCorrection] else
    APicture.Options := APicture.Options - [poCaseCorrection];
  if chCaseIns.Checked then
    APicture.Options := APicture.Options + [poCaseInsensitive] else
    APicture.Options := APicture.Options - [poCaseInsensitive];
end;

procedure Tvg2MaskEditorForm.AssignListItem(Item: TListItem; const APicture, ADescription: string; AFlags: Integer);
begin
  Item.Caption := APicture;
  Item.SubItems.Clear;
  Item.SubItems.Add(ADescription);
  Item.Data := Pointer(AFlags);
end;

procedure Tvg2MaskEditorForm.UpdateStates;
begin
  cmInsert.Enabled := Assigned(FIniFile);
  cmEdit.Enabled := Assigned(FIniFile) and Assigned(lvSamples.Selected);
  cmDelete.Enabled := cmEdit.Enabled;
end;

procedure Tvg2MaskEditorForm.FormShow(Sender: TObject);
begin
  inherited;
  edMask.Text := FPicture.PictureMask;
  chAutoFill.Checked := poAutoFill in FPicture.Options;
  chCaseCorrection.Checked := poCaseCorrection in FPicture.Options;
  chCaseIns.Checked := poCaseInsensitive in FPicture.Options;
  UpdateStatus;
  LoadMasks;
  UpdateStates;
end;

procedure Tvg2MaskEditorForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SaveMasks;
end;

procedure Tvg2MaskEditorForm.edTestEnter(Sender: TObject);
begin
  AssignPicture(edTest.Picture);
  UpdateStatus;
end;

procedure Tvg2MaskEditorForm.UpdateStatus;
var
  Value, AutoFill: string;
begin
  Value := edTest.Text;
  if edTest.Picture.IsInputValid(Value, AutoFill, False) then
    lbStatus.Caption := SPictureMaskValid else
    lbStatus.Caption := SPictureMaskInvalid;
end;

procedure Tvg2MaskEditorForm.edTestChange(Sender: TObject);
begin
  UpdateStatus;
end;

procedure Tvg2MaskEditorForm.LoadMasks;
var
  I, ReadCount: Integer;
  Picture, Descr: string;
  Flags: Integer;
  ListItem: TListItem;
begin
  if not Assigned(FIniFile) then Exit;

  ReadCount := FIniFile.ReadInteger(SPictures, SCount, 0);
  if ReadCount = 0 then Exit;

  with lvSamples.Items do
  begin
    BeginUpdate;
    try
      for I := 0 to ReadCount - 1 do
      begin
        Picture := FIniFile.ReadString(SPictures, Format(SPicture, [I]), '');
        Descr := FIniFile.ReadString(SPictures, Format(SDescription, [I]), '');
        Flags := FIniFile.ReadInteger(SPictures, Format(SFlags, [I]), 0);
        if Picture <> '' then
        begin
          ListItem := Add;
          AssignListItem(ListItem, Picture, Descr, Flags);
        end;
      end;
    finally
      EndUpdate;
    end;
  end;
end;

procedure Tvg2MaskEditorForm.SaveMasks;
var
  I: Integer;
  ListItem: TListItem;
begin
  if not (Assigned(FIniFile) and FModified) then Exit;
  FIniFile.EraseSection(SPictures);
  if lvSamples.Items.Count > 0 then
  begin
    FIniFile.WriteInteger(SPictures, SCount, lvSamples.Items.Count);
    for I := 0 to lvSamples.Items.Count - 1 do
    begin
      ListItem := lvSamples.Items[I];
      FIniFile.WriteString(SPictures, Format(SPicture, [I]), ListItem.Caption);
      FIniFile.WriteString(SPictures, Format(SDescription, [I]), ListItem.SubItems[0]);
      FIniFile.WriteInteger(SPictures, Format(SFlags, [I]), Integer(ListItem.Data));
    end;
  end;
end;

procedure Tvg2MaskEditorForm.lvSamplesClick(Sender: TObject);
var
  Item: TListItem;
  Flags: Integer;
begin
  Item := lvSamples.Selected;
  if Assigned(Item) then
  begin
    Flags := Integer(Item.Data);
    edMask.Text := Item.Caption;
    chAutoFill.Checked := Flags and 1 = 1;
    chCaseCorrection.Checked := Flags and 2 = 2;
    chCaseIns.Checked := Flags and 4 = 4;
  end;
  UpdateStates;
end;

procedure Tvg2MaskEditorForm.cmEditClick(Sender: TObject);
var
  Item: TListItem;
  Picture, Description: string;
  Flags: Integer;
  VarStore: Variant;
  IsNew, AutoFill, CaseCorrection, CaseInsensitive: Boolean;
begin
  IsNew := TComponent(Sender).Tag = 0;

  VarStore := VarArrayOf([
    VarArrayOf(['Picture', edMask.Text]),
    VarArrayOf(['AutoFill', chAutoFill.Checked]),
    VarArrayOf(['CaseCorrection', chCaseCorrection.Checked]),
    VarArrayOf(['CaseInsensitive', chCaseIns.Checked]),
    VarArrayOf(['IsNew', IsNew])
  ]);

  if not IsNew then
    VarStoreSet(VarStore, 'Description', lvSamples.Selected.SubItems[0]);

  if ShowDialog(Tvg2MaskEditDialogForm, VarStore) then
  begin
    Picture := VarStoreGet(VarStore, 'Picture');
    Description := VarStoreGet(VarStore, 'Description');
    AutoFill := VarStoreGet(VarStore, 'AutoFill');
    CaseCorrection := VarStoreGet(VarStore, 'CaseCorrection');
    CaseInsensitive := VarStoreGet(VarStore, 'CaseInsensitive');

    Flags := 1 * ord(AutoFill) or 2 * ord(CaseCorrection) or 4 * ord(CaseInsensitive);

    if IsNew then
      Item := lvSamples.Items.Add else
      Item := lvSamples.Selected;
    AssignListItem(Item, Picture, Description, Flags);
    Modified;
    lvSamples.Selected := Item;
    lvSamplesClick(nil);
  end;
end;

procedure Tvg2MaskEditorForm.cmDeleteClick(Sender: TObject);
begin
  if MessageDlg(SConfirmDeleteRecord, mtConfirmation, [mbYes, mbNo], 0) = mrYes then
  begin
    lvSamples.Selected.Free;
    Modified;
    UpdateStates;
  end;
end;

procedure Tvg2MaskEditorForm.Modified;
begin
  FModified := True;
end;

end.
